/*========================================================================
 * $RCSfile: PatternEditor.java,v $
 *
 * $Author: liddle $
 * $Revision: 1.8 $
 * $Date: 1998/08/06 16:31:04 $
 *
 * $Log: PatternEditor.java,v $
 * Revision 1.8  1998/08/06  16:31:04  liddle
 * Lots 'o changes!  New architecture is now in place.
 *
 * Revision 1.7  1998/06/27  15:52:38  liddle
 * About to reorganize for MVC architecture.
 *
 * Revision 1.6  1998/06/24  16:56:00  liddle
 * Implemented Cut/Copy/Paste.  Note that the current JDK 1.1 has
 * problems with the system clipboard.  This clipboard only works
 * for this application -- can't share clipped info with others.
 *
 * Added Pattern and PatternVector classes to do the
 * data model right.
 *
 * Added About and Help dialogs.
 *
 * Modified the color palette.  This could still use some work.
 *
 * Revision 1.5  1998/06/22  16:37:01  liddle
 * Another major development checkpoint.  More functions are
 * implemented.  Still lacking: clipboard interface for
 * Cut/Copy/Paste.  Need to put highlighting into a background
 * thread for efficiency.  Need to clean up the About box and
 * write a help system.  Need to take action on Undo/Redo
 * (right now it just logs the actions and prints a string on
 * the console indicating what to undo or redo).  Also, we
 * need a final color scheme for the highlighting.  Finally,
 * when we're all done we should package this thing as a JRE
 * that is easy to distribute.
 *
 * Revision 1.4  1998/06/16  23:43:39  liddle
 * Many changes.  Much of the user interface is working.
 * Still working on lots of the application logic.
 *
 * Revision 1.3  1998/06/16  00:48:57  liddle
 * Main aspects of UI are working.
 * Now ready to implement menu functions.
 *
 * Revision 1.2  1998/06/13  21:22:39  liddle
 * About to change pattern list from JList to JTable.
 *
 * Revision 1.1  1998/06/12  20:31:06  liddle
 * Initial revision
 *
 *------------------------------------------------------------------------
 * Pattern Editor
 * Data Extraction Research Group
 * School of Accountancy and Information Systems
 * Marriott School of Management
 * 540 TNRB
 * Brigham Young University
 * Provo, UT 84602-3068
 * liddle@byu.edu
 *
 * Copyright (C) 1998, Board of Trustees of Brigham Young University
 *
 * Pattern Editor software, both binary and source (hereafter, Software)
 * is copyrighted by The Board of Trustees of Brigham Young University
 * (BYU), and ownership remains with BYU.
 *
 * BYU grants you (hereafter, Licensee) a license to use the Software
 * for academic, research and internal business purposes only, without a
 * fee.  Licensee may distribute the binary and source code (if released)
 * to third parties provided that the copyright notice and this statement
 * appears on all copies and that no charge is associated with such
 * copies.
 *
 * Licensee may not make derivative works without first arranging an
 * additional license agreement with BYU.
 *
 * Any Licensee wishing to make commercial use of the Software should
 * contact BYU to negotiate an appropriate license for such
 * commercial use.  Commercial use includes (1) integration of all or
 * part of the source code into a product for sale or license by or on
 * behalf of Licensee to third parties, or (2) distribution of the binary
 * code or source code to third parties that need it to utilize a
 * commercial product sold or licensed by or on behalf of Licensee.
 *
 * BYU MAKES NO REPRESENTATIONS ABOUT THE SUITABILITY OF THIS SOFTWARE
 * FOR ANY PURPOSE.  IT IS PROVIDED "AS IS" WITHOUT EXPRESS OR IMPLIED
 * WARRANTY.  BYU SHALL NOT BE LIABLE FOR ANY DAMAGES SUFFERED BY THE
 * USERS OF THIS SOFTWARE.
 *
 * By using or copying this Software, Licensee agrees to abide by the
 * copyright law and all other applicable laws of the United States
 * including, but not limited to, export control laws, and the terms
 * of this license.  BYU shall have the right to terminate this license
 * immediately by written notice upon Licensee's breach of, or
 * non-compliance with, any of its terms.  Licensee may be held legally
 * responsible for any copyright infringement that is caused or
 * encouraged by Licensee's failure to abide by the terms of this license.
 *
 * Comments and questions are welcome and can be sent to liddle@byu.edu.
 */
/*------------------------------------------------------------------------
 *			IMPORTS
 */
import java.awt.*;
import java.awt.datatransfer.*;
import java.awt.event.*;
import java.beans.*;
import java.util.*;
import java.io.*;
import java.net.*;

import javax.swing.*;
import javax.swing.event.*;
import javax.swing.text.*;
import javax.swing.border.*;
import javax.swing.table.*;

/*========================================================================
 *			CLASS PatternEditor
 */
public class PatternEditor extends JFrame {

/*------------------------------------------------------------------------
 *			PUBLIC CONSTANTS
 */
public static final int		CASE_COL_NUM = 0,
				ID_COL_NUM = 1,
				PATTERN_COL_NUM = 2,
				NUM_COLUMNS = 3;
public static final String	REORDER_BOTTOM = "Pattern to Bottom",
				REORDER_DOWN = "Pattern Down",
				REORDER_TOP = "Pattern to Top",
				REORDER_UP = "Pattern Up";

/*------------------------------------------------------------------------
 *			PRIVATE CONSTANTS
 */
private static final String	APP_TITLE = "Pattern Editor",
				WELCOME_MESSAGE =
			" Welcome to the Report Pattern Editor Utility." +
  			"  Open pattern and report files to begin.";

private static final int	MULTIPLE_INTERVAL_SELECTION = 2,
				NUM_COLORS = 16,
				BRT = 255,
				HI  = 200,
				MID = 160,
				LO  = 128;

private static final boolean	CUT = true,
				DELETE = false;

private static final DataFlavor	PATTERN_FLAVOR = new DataFlavor(
				  PatternVector.class, "PatternVector");

/*------------------------------------------------------------------------
 *			ATTRIBUTES
 */
private boolean			pe_constructed;
private	int			row_height;
private Font			default_font,
				fixed_width_font,
				subtitle_font,
				title_font;
private Frame			pview;
private PatternTable		pattern_view;
private JLabel			status_bar;
private ReportTextPane		edit_pane;
private JSplitPane		split_pane;
private JScrollPane		scroll_list,
				scroll_text;
private String			path;
private Clipboard		clip = new Clipboard(APP_TITLE);
			//Toolkit.getDefaultToolkit().getSystemClipboard();

private com.oroinc.text.regex.Perl5Matcher
			matcher = new com.oroinc.text.regex.Perl5Matcher();
private Stack			undo_log = new Stack(),
				redo_log = new Stack();
private String			clip_data = "";
private Color []		colors =
				{
				  new Color(BRT, HI, HI),
				  new Color(HI, BRT, HI),
				  new Color(HI, HI, BRT),
				  new Color(BRT, MID, MID),
				  new Color(MID, BRT, BRT),
				  new Color(BRT, BRT, MID),
				  new Color(BRT, HI, BRT),
				  new Color(HI, MID, MID),
				  new Color(MID, HI, MID),
				  new Color(MID, MID, HI),
				  new Color(HI, HI, MID),
				  new Color(HI, MID, HI),
				  new Color(MID, HI, HI),
				  new Color(BRT, BRT, LO),
				  new Color(LO, BRT, LO),
				  new Color(BRT, MID, BRT),
				  new Color(HI, HI, HI)
				};

private Highlighter		hlite;
private Highlighter.HighlightPainter []
		painters = new Highlighter.HighlightPainter[NUM_COLORS];

private ActionListener		new_action, open_action, save_action,
				save_as_action, view_report_action,
				exit_action, undo_action, redo_action,
				cut_action, copy_action, paste_action,
				delete_action, reorder_action,
				help_action, about_action;
private TableModelListener	plist_update;
private JButton			tb_new, tb_open, tb_save, tb_view_report,
				tb_cut, tb_copy, tb_paste, tb_undo,
				tb_redo, tb_delete, tb_to_top, tb_up,
				tb_down, tb_to_bottom, tb_help;
private JCheckBox		tb_highlight;
private JMenuItem		mi_new, mi_open, mi_save, mi_save_as,
				mi_view_report, mi_exit, mi_undo, mi_redo,
				mi_cut, mi_copy, mi_paste, mi_delete,
				mi_to_top, mi_up, mi_down, mi_to_bottom,
				mi_help, mi_about;
private JDialog			about_dialog,
				help_dialog;

private PatternEditorModel	model;

/*------------------------------------------------------------------------
 *			CONSTRUCTORS
 */
public
PatternEditor(String pfile, String rfile)
{
  super(APP_TITLE);

  PatternCellEditor	ce;
  JPanel		main_pane,
			top_pane;
  JViewport		vp;
  String		cur_dir;

  pe_constructed = false;

  pview = this;
  status_bar = new JLabel(WELCOME_MESSAGE);

  try {
    model = new PatternEditorModel(pfile, rfile, this);
  } catch (IOException e) {
  }

  path = model.Get_Pattern_Path();

  default_font = new Font("Dialog", Font.PLAIN, 12);
  fixed_width_font = new Font("Courier", Font.PLAIN, 12);
  subtitle_font = new Font("Dialog", Font.BOLD, 12);
  title_font = new Font("Dialog", Font.BOLD, 24);
  setFont(default_font);

  main_pane = new JPanel();
  setContentPane(main_pane);
  main_pane.setLayout(new BorderLayout());

  // Create all the action listeners we'll need to do any work.
  Create_Listeners();

  // Top zone contains menu bar above tool bar.
  top_pane = new JPanel();
  top_pane.setLayout(new BorderLayout());
  top_pane.add(Create_Menu_Bar(), BorderLayout.NORTH);
  top_pane.add(Create_Tool_Bar(), BorderLayout.CENTER);
  Enable_Clipboard();
  main_pane.add(top_pane, BorderLayout.NORTH);

  // Bottom zone contains status bar (implemented as a label).
  status_bar.setBorder(new BevelBorder(BevelBorder.LOWERED));
  main_pane.add(status_bar, BorderLayout.SOUTH);

  // Middle zone contains pattern list box left of text viewer.
  model.Add_PatternTableModel_Listener(plist_update);
  pattern_view = new PatternTable(model.Get_PatternTableModel(), this);
  ce = new PatternCellEditor(new JTextField());
  ce.setFont(fixed_width_font);
  pattern_view.setDefaultEditor(String.class, ce);
  pattern_view.setSelectionMode(MULTIPLE_INTERVAL_SELECTION);
  pattern_view.setAutoResizeMode(JTable.AUTO_RESIZE_LAST_COLUMN);
  pattern_view.getTableHeader().setReorderingAllowed(false);
  pattern_view.setFont(fixed_width_font);
  pattern_view.setDefaultRenderer(String.class, new PatternCellRenderer());
  pattern_view.setPreferredScrollableViewportSize(
					new Dimension(1024, 356));
  pattern_view.setMinimumSize(new Dimension(100, 100));
  row_height = pattern_view.getRowHeight() +
  	       pattern_view.getIntercellSpacing().height;
  ColorCheckRenderer cr = new ColorCheckRenderer();
  cr.setHorizontalAlignment(JLabel.CENTER);
  pattern_view.setDefaultRenderer(Boolean.class, cr);

  scroll_list = new JScrollPane(pattern_view,
			ScrollPaneConstants.VERTICAL_SCROLLBAR_ALWAYS,
			ScrollPaneConstants.HORIZONTAL_SCROLLBAR_NEVER);
  scroll_list.setMinimumSize(new Dimension(100, 100));
  vp = scroll_list.getViewport();
  vp.setScrollMode(JViewport.BLIT_SCROLL_MODE);

  edit_pane = new ReportTextPane(model.Get_Document());
  edit_pane.setFont(default_font);
  hlite = edit_pane.getHighlighter();

  for (int i = 0; i < NUM_COLORS; i++)
    painters[i] =
    	new DefaultHighlighter.DefaultHighlightPainter(colors[i]);

  scroll_text = new JScrollPane(edit_pane);
  scroll_text.setMinimumSize(new Dimension(100, 100));

  split_pane = new JSplitPane(JSplitPane.VERTICAL_SPLIT, true,
				scroll_list, scroll_text);
  split_pane.setDividerSize(4);
  split_pane.setCursor(
		Cursor.getPredefinedCursor(Cursor.N_RESIZE_CURSOR));
  main_pane.add(split_pane, BorderLayout.CENTER);

  Format_Title();

} /* PatternEditor constructor */

/*------------------------------------------------------------------------
 *			PRIVATE METHODS
 */
private void
Action_Copy(boolean enable)
{
  int		row_cnt = pattern_view.getRowCount(),
		sel_cnt = pattern_view.getSelectedRowCount();
  int []	sel_rows = pattern_view.getSelectedRows();

  if (sel_cnt > 0 && sel_rows[sel_cnt - 1] + 1 >= row_cnt)
    --sel_cnt;
  if (sel_cnt > 0) {
    PatternVector	to_copy;
    PatternSelection	pat_sel;

    to_copy = model.Clone_Patterns(sel_rows, sel_cnt);
    //clip = Toolkit.getDefaultToolkit().getSystemClipboard();
    pat_sel = new PatternSelection(to_copy, PATTERN_FLAVOR);
    clip.setContents(pat_sel, pat_sel);
    if (enable)
      Enable_Clipboard();
  }

} /* Action_Copy */

private void
Action_Cut(boolean how)
{
  int		sel_cnt = pattern_view.getSelectedRowCount();
  int []	sel_rows = pattern_view.getSelectedRows();
  String	undo;

  if (sel_cnt > 0) {
    undo = "DELETE_SELECTION " + sel_cnt + "\n";
    if (how == CUT)
      Action_Copy(false);
    for (int i = sel_cnt - 1; i >= 0; i--)
      undo = undo + sel_rows[i] + " " +
      	     model.Get_Pattern(sel_rows[i]) + "\n";
    model.Remove_Patterns(sel_rows, sel_cnt);
    Log_Undoable_Action(undo);
    pattern_view.clearSelection();
    Enable_Clipboard();
  }

} /* Action_Cut */

private void
Action_Paste()
{
  Transferable	t = clip.getContents(this);
  PatternVector	pat_sel;

  if (t == null)
    return;
  try {
    pat_sel = (PatternVector)t.getTransferData(PATTERN_FLAVOR);
    if (pat_sel != null && pat_sel.size() > 0) {
      int	row_cnt = pattern_view.getRowCount(),
		sel,
		sel_cnt = pattern_view.getSelectedRowCount();
      int []	sel_rows = pattern_view.getSelectedRows();
      String	undo;

      if (sel_cnt <= 0)
        sel = row_cnt - 1;
      else
        sel = sel_rows[0];
      undo = "PASTE " + pat_sel.size() + "\n";
      for (int i = pat_sel.size() - 1; i >= 0; i--)
	undo = sel + pat_sel.Get(i).toString() + "\n";
      model.Insert_Patterns(pat_sel, sel);
      pattern_view.clearSelection();
      pattern_view.setRowSelectionInterval(sel, sel + pat_sel.size() - 1);
      Log_Undoable_Action(undo);
    }
  } catch (UnsupportedFlavorException e) {
    // don't paste if we get an exception
  } catch (IOException e) {
    // don't paste if we get an exception
  }

} /* Action_Paste */

private JLabel
Add_Label(String str, JPanel panel)
{
  return (Add_Label(str, panel, default_font));

} /* Add_Label */

private JLabel
Add_Label(String str, JPanel panel, Font font)
{
  JLabel	label = new JLabel(str, SwingConstants.CENTER);

  label.setFont(font);
  label.setForeground(Color.black);
  panel.add(label);
  return (label);

} /* Add_Label */

private JButton
Add_Tool_Bar_Item(JToolBar tb, String name,
		  boolean enabled, ActionListener al)
{
  try {
    Class c = Class.forName("PatternEditor");
    URL u = c.getResource("images/" + name + ".gif");
    JButton b = (JButton)tb.add( new JButton(new ImageIcon(u, name)) );
    b.setToolTipText(name);
    b.setMargin(new Insets(0, 0, 0, 0));
    b.setEnabled(enabled);
    b.addActionListener(al);
    return (b);
  } catch (Exception e) {
    System.err.println("Exception creating button. " + e);
    /*
     * No button will be generated if there is an exception.
     * Fail silently.
     */
  }
  return (null);

} /* Add_Tool_Bar_Item */

private JCheckBox
Add_Tool_Bar_Toggle(JToolBar tb, String name,
		    boolean enabled, ActionListener al)
{
  try {
    Class c = Class.forName("PatternEditor");
    URL u = c.getResource("images/" + name + ".gif");
    JCheckBox b = (JCheckBox)tb.add(new JCheckBox(name));
    b.setToolTipText(name);
    b.setMargin(new Insets(0, 0, 0, 0));
    b.setEnabled(enabled);
    if (al != null)
      b.addActionListener(al);
    return (b);
  } catch (Exception e) {
    /*
     * No button will be generated if there is an exception.
     * Fail silently.
     */
  }
  return (null);

} /* Add_Tool_Bar_Item */

private int
Confirm_Save_Changes(int options)
{
  if (model.Is_Modified()) {
    int	result;
	
    result = JOptionPane.showConfirmDialog(getContentPane(),
			"Save changes to " + model.Get_Pattern_Filename() + "?",
			APP_TITLE, options,
			JOptionPane.WARNING_MESSAGE);
    if (result == JOptionPane.CANCEL_OPTION)
      return (result);
    if (result == JOptionPane.YES_OPTION) {
      if (Save_Changes() == JOptionPane.CANCEL_OPTION)
	return (JOptionPane.CANCEL_OPTION);
    }
  }
  return (JOptionPane.YES_OPTION);

} /* Confirm_Save_Changes */

private void
Create_Listeners()
{
  new_action = new ActionListener() {
    public void actionPerformed(ActionEvent e) {
      if (Confirm_Save_Changes(JOptionPane.YES_NO_CANCEL_OPTION) !=
	  JOptionPane.CANCEL_OPTION) {
	model.Set_Pattern_Filename(PatternEditorModel.UNTITLED_NAME);
	Load_Patterns();
      }
    }
  };

  open_action = new ActionListener() {
    public void actionPerformed(ActionEvent e) {
      JFileChooser	chooser;
      int		result;

      if (Confirm_Save_Changes(JOptionPane.YES_NO_CANCEL_OPTION) ==
	  JOptionPane.CANCEL_OPTION)
	return;
      chooser = new JFileChooser();
      chooser.setCurrentDirectory(new File(model.Get_Pattern_Path()));
      chooser.setSelectedFile(new File(model.Get_Pattern_Filename()));
      chooser.setDialogTitle(APP_TITLE);
      result = chooser.showOpenDialog(getContentPane());
      if (result == JFileChooser.CANCEL_OPTION)
        return;
      model.Set_Pattern_Path(chooser.getCurrentDirectory().getName());
      model.Set_Pattern_Filename(chooser.getSelectedFile().getName());
      Load_Patterns();
    }
  };

  save_action = new ActionListener() {
    public void actionPerformed(ActionEvent e) {
      Save_Changes();
    }
  };

  save_as_action = new ActionListener() {
    public void actionPerformed(ActionEvent e) {
      Save_Changes_As();
    }
  };

  view_report_action = new ActionListener() {
    public void actionPerformed(ActionEvent e) {
      JFileChooser	chooser;
      int		result;

      chooser = new JFileChooser();
      chooser.setCurrentDirectory(new File(model.Get_Report_Path()));
      chooser.setSelectedFile(new File(model.Get_Report_Filename()));
      chooser.setDialogTitle("View Report");
      result = chooser.showOpenDialog(getContentPane());
      if (result != JFileChooser.CANCEL_OPTION) {
        model.Set_Report_Path(chooser.getCurrentDirectory().getName());
	model.Set_Report_Filename(chooser.getSelectedFile().getName());
	model.Load_Report();
	scroll_text.getViewport().setViewPosition(new Point(0, 0));
      	hlite.removeAllHighlights();
      }
    }
  };

  exit_action = new ActionListener() {
    public void actionPerformed(ActionEvent e) {
      if (Confirm_Save_Changes(JOptionPane.YES_NO_CANCEL_OPTION) !=
	  JOptionPane.CANCEL_OPTION)
	System.exit(0);
    }
  };

  undo_action = new ActionListener() {
    public void actionPerformed(ActionEvent e) {
      if (!undo_log.isEmpty()) {
        redo_log.push(undo_log.peek());
        System.out.println("Undo " + undo_log.pop());
      }
      Enable_Undo_Redo();
    }
  };

  redo_action = new ActionListener() {
    public void actionPerformed(ActionEvent e) {
      if (!redo_log.isEmpty()) {
        undo_log.push(redo_log.peek());
        System.out.println("Redo " + redo_log.pop());
      }
      Enable_Undo_Redo();
    }
  };

  cut_action = new ActionListener() {
    public void actionPerformed(ActionEvent e) {
      Action_Cut(CUT);
    }
  };

  copy_action = new ActionListener() {
    public void actionPerformed(ActionEvent e) {
      Action_Copy(true);
    }
  };

  paste_action = new ActionListener() {
    public void actionPerformed(ActionEvent e) {
      Action_Paste();
    }
  };

  delete_action = new ActionListener() {
    public void actionPerformed(ActionEvent e) {
      Action_Cut(DELETE);
    }
  };

  reorder_action = new ActionListener() {
    public void actionPerformed(ActionEvent e) {
      Reorder_Patterns(e);
    }
  };

  help_action = new ActionListener() {
    public void actionPerformed(ActionEvent e) {
      Show_Help_Dialog();
    }
  };

  about_action = new ActionListener() {
    public void actionPerformed(ActionEvent e) {
      Show_About_Dialog();
    }
  };

  plist_update = new TableModelListener() {
    public void tableChanged(TableModelEvent e) {
      Update_Highlights();
    }
  };

} /* Create_Listeners */

private JMenuBar
Create_Menu_Bar()
{
  JMenuBar	mb = new JMenuBar();
  JMenu		menu;

  // File Menu
  menu = (JMenu)mb.add(new JMenu("File"));
  menu.setMnemonic('F');
  mi_new = (JMenuItem) menu.add(new JMenuItem("New"));
  mi_new.setMnemonic('N');
  mi_new.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_N,
					       Event.CTRL_MASK));
  mi_new.setEnabled(true);
  mi_new.addActionListener(new_action);

  mi_open = (JMenuItem) menu.add(new JMenuItem("Open..."));
  mi_open.setMnemonic('O');
  mi_open.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_O,
					        Event.CTRL_MASK));
  mi_open.setEnabled(true);
  mi_open.addActionListener(open_action);

  mi_save = (JMenuItem) menu.add(new JMenuItem("Save"));
  mi_save.setMnemonic('S');
  mi_save.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_S,
						Event.CTRL_MASK));
  mi_save.setEnabled(false);
  mi_save.addActionListener(save_action);

  mi_save_as = (JMenuItem) menu.add(new JMenuItem("Save As..."));
  mi_save_as.setMnemonic('A');
  mi_save_as.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_A,
						   Event.CTRL_MASK));
  mi_save_as.setEnabled(false);
  mi_save_as.addActionListener(save_as_action);

  menu.add(new JSeparator());
  mi_view_report = (JMenuItem) menu.add(new JMenuItem("View Report..."));
  mi_view_report.setMnemonic('R');
  mi_view_report.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_R,
						       Event.CTRL_MASK));
  mi_view_report.setEnabled(true);
  mi_view_report.addActionListener(view_report_action);

  menu.add(new JSeparator());
  mi_exit = (JMenuItem) menu.add(new JMenuItem("Exit"));
  mi_exit.setMnemonic('x');
  mi_exit.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_F4,
						Event.ALT_MASK));
  mi_exit.addActionListener(exit_action);

  // Edit Menu
  menu = (JMenu)mb.add(new JMenu("Edit"));
  menu.setMnemonic('E');
  mi_undo = (JMenuItem) menu.add(new JMenuItem("Undo"));
  mi_undo.setMnemonic('U');
  mi_undo.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_Z,
						Event.CTRL_MASK));
  mi_undo.setEnabled(false);
  mi_undo.addActionListener(undo_action);

  mi_redo = (JMenuItem) menu.add(new JMenuItem("Redo"));
  mi_redo.setMnemonic('R');
  mi_redo.setEnabled(false);
  mi_redo.addActionListener(redo_action);

  menu.add(new JSeparator());
  mi_cut = (JMenuItem) menu.add(new JMenuItem("Cut"));
  mi_cut.setMnemonic('t');
  mi_cut.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_X,
					       Event.CTRL_MASK));
  mi_cut.setEnabled(false);
  mi_cut.addActionListener(cut_action);

  mi_copy = (JMenuItem) menu.add(new JMenuItem("Copy"));
  mi_copy.setMnemonic('C');
  mi_copy.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_C,
						Event.CTRL_MASK));
  mi_copy.setEnabled(false);
  mi_copy.addActionListener(copy_action);

  mi_paste = (JMenuItem) menu.add(new JMenuItem("Paste"));
  mi_paste.setMnemonic('P');
  mi_paste.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_V,
						 Event.CTRL_MASK));
  mi_paste.setEnabled(false);
  mi_paste.addActionListener(paste_action);

  mi_delete = (JMenuItem) menu.add(new JMenuItem("Delete"));
  mi_delete.setMnemonic('D');
  mi_delete.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_DELETE, 0));
  mi_delete.setEnabled(false);
  mi_delete.addActionListener(delete_action);

  // Order Menu
  menu = (JMenu)mb.add(new JMenu("Order"));
  menu.setMnemonic('O');
  mi_to_top = (JMenuItem) menu.add(new JMenuItem(REORDER_TOP));
  mi_to_top.setMnemonic('T');
  mi_to_top.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_T,
						  Event.CTRL_MASK));
  mi_to_top.setEnabled(false);
  mi_to_top.addActionListener(reorder_action);

  mi_up = (JMenuItem) menu.add(new JMenuItem(REORDER_UP));
  mi_up.setMnemonic('U');
  mi_up.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_U,
					      Event.CTRL_MASK));
  mi_up.setEnabled(false);
  mi_up.addActionListener(reorder_action);

  mi_down = (JMenuItem) menu.add(new JMenuItem(REORDER_DOWN));
  mi_down.setMnemonic('D');
  mi_down.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_D,
						Event.CTRL_MASK));
  mi_down.setEnabled(false);
  mi_down.addActionListener(reorder_action);

  mi_to_bottom = (JMenuItem) menu.add(new JMenuItem(REORDER_BOTTOM));
  mi_to_bottom.setMnemonic('B');
  mi_to_bottom.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_B,
						     Event.CTRL_MASK));
  mi_to_bottom.setEnabled(false);
  mi_to_bottom.addActionListener(reorder_action);

  // Help Menu
  menu = (JMenu)mb.add(new JMenu("Help"));
  menu.setMnemonic('H');
  mi_help = (JMenuItem) menu.add(new JMenuItem("Help Topics"));
  mi_help.setMnemonic('H');
  mi_help.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_H,
						Event.CTRL_MASK));
  mi_help.setEnabled(true);
  mi_help.addActionListener(help_action);

  menu.add(new JSeparator());
  mi_about = (JMenuItem) menu.add(new JMenuItem("About " + APP_TITLE));
  mi_about.setMnemonic('A');
  mi_about.setEnabled(true);
  mi_about.addActionListener(about_action);

  return (mb);

} /* Create_Menu_Bar */

private JToolBar
Create_Tool_Bar()
{
  JToolBar	tb = new JToolBar();

  tb.setFloatable(false);
  tb_new = Add_Tool_Bar_Item(tb, "New", true, new_action);
  tb_open = Add_Tool_Bar_Item(tb, "Open", true, open_action);
  tb_save = Add_Tool_Bar_Item(tb, "Save", false, save_action);
  tb.addSeparator();
  tb_view_report = Add_Tool_Bar_Item(tb, "View Report", true,
				     view_report_action);
  tb.addSeparator();
  tb_cut = Add_Tool_Bar_Item(tb, "Cut", false, cut_action);
  tb_copy = Add_Tool_Bar_Item(tb, "Copy", false, copy_action);
  tb_paste = Add_Tool_Bar_Item(tb, "Paste", false, paste_action);
  tb.addSeparator();
  tb_undo = Add_Tool_Bar_Item(tb, "Undo", false, undo_action);
  tb_redo = Add_Tool_Bar_Item(tb, "Redo", false, redo_action);
  tb.addSeparator();
  tb_delete = Add_Tool_Bar_Item(tb, "Delete", false, delete_action);
  tb.addSeparator();
  tb_to_top = Add_Tool_Bar_Item(tb, REORDER_TOP, false, reorder_action);
  tb_up = Add_Tool_Bar_Item(tb, REORDER_UP, false, reorder_action);
  tb_down = Add_Tool_Bar_Item(tb, REORDER_DOWN, false, reorder_action);
  tb_to_bottom = Add_Tool_Bar_Item(tb, REORDER_BOTTOM, false,
				   reorder_action);
  tb.addSeparator();
  tb_help = Add_Tool_Bar_Item(tb, "Help", true, help_action);
  tb.addSeparator();
  tb_highlight = Add_Tool_Bar_Toggle(tb, "Highlight Matches", true, null);

  return (tb);

} /* Create_Tool_Bar */

public void
Enable_Clipboard()
{
  //Clipboard	clip = Toolkit.getDefaultToolkit().getSystemClipboard();
  //if (clip == null)
  //  return;

  Transferable	t = clip.getContents(this);
  boolean	enabled = false;

  if (t == null)
    return;
  try {
    if (t.getTransferData(PATTERN_FLAVOR) != null)
      enabled = true;
  } catch (UnsupportedFlavorException e) {
    // don't enable paste if we get an exception
  } catch (IOException e) {
    // don't enable paste if we get an exception
  }
  mi_paste.setEnabled(enabled);
  tb_paste.setEnabled(enabled);

} /* Enable_Clipboard */

public void
Enable_Reorder()
{
  int		row_cnt = pattern_view.getRowCount(),
		sel_cnt = pattern_view.getSelectedRowCount();
  int []	sel_rows = pattern_view.getSelectedRows();
  boolean	bottom = false,
  		down = false,
		up = false,
		top = false;

  if (sel_cnt > 0 && sel_rows[sel_cnt - 1] + 1 >= row_cnt)
    --sel_cnt;
  if (sel_cnt >= 1) {
    if (sel_rows[0] > 0) {
      top = true;
      up = true;
    }
    if (sel_rows[sel_cnt - 1] + 1 < row_cnt - 1) {
      bottom = true;
      down = true;
    }
  }
  mi_to_top.setEnabled(top);
  tb_to_top.setEnabled(top);
  mi_up.setEnabled(up);
  tb_up.setEnabled(up);
  mi_down.setEnabled(down);
  tb_down.setEnabled(down);
  mi_to_bottom.setEnabled(bottom);
  tb_to_bottom.setEnabled(bottom);

} /* Enable_Reorder */

public void
Enable_Save()
{
  tb_save.setEnabled(model.Is_Modified());
  mi_save.setEnabled(model.Is_Modified());
  if (model.Is_Modified() || model.Is_PatternVector_Untitled())
    mi_save_as.setEnabled(true);
  else
    mi_save_as.setEnabled(model.Is_PatternVector_Untitled());

} /* Enable_Save */

public void
Enable_Selected(boolean enabled)
{
  tb_cut.setEnabled(enabled);
  tb_copy.setEnabled(enabled);
  tb_delete.setEnabled(enabled);

  mi_cut.setEnabled(enabled);
  mi_copy.setEnabled(enabled);
  mi_delete.setEnabled(enabled);

  if (!enabled) {
    tb_to_top.setEnabled(enabled);
    tb_up.setEnabled(enabled);
    tb_down.setEnabled(enabled);
    tb_to_bottom.setEnabled(enabled);
    mi_to_top.setEnabled(enabled);
    mi_up.setEnabled(enabled);
    mi_down.setEnabled(enabled);
    mi_to_bottom.setEnabled(enabled);
  } else {
    Enable_Reorder();
  }

} /* Enable_Selected */

private void
Enable_Undo_Redo()
{
  tb_undo.setEnabled(undo_log.size() > 0);
  mi_undo.setEnabled(undo_log.size() > 0);
  tb_redo.setEnabled(redo_log.size() > 0);
  mi_redo.setEnabled(redo_log.size() > 0);

} /* Enable_Undo_Redo */

private void
Format_Title()
{
  String	title = APP_TITLE + " - ";

  if (model.Is_PatternVector_Untitled())
    title = title + PatternEditorModel.UNTITLED_NAME;
  else
    title = title + model.Get_Pattern_Path() +
    	    model.Get_Pattern_Filename();
  if (model.Is_Modified())
    title = title + "  [Modified]";
  else
    title = title + "  [Unmodified]";
  setTitle(title);

} /* Format_Title */

private void
Load_Patterns()
{
  if (model.Load_Patterns())
    Set_StatusBar("Unable to load or parse patterns file.");
  else
    Set_StatusBar("");
  Format_Title();
  pattern_view.clearSelection();
  pattern_view.setEditingColumn(0);
  pattern_view.setEditingRow(0);
  Enable_Save();

} /* Load_Patterns */

private void
Log_Undoable_Action(String undo)
{
  if (!model.Is_Modified()) {
    model.Set_Modified(true);
    Format_Title();
  }
  undo_log.push(undo);
  redo_log.removeAllElements();
  Enable_Undo_Redo();
  Enable_Save();

} /* Log_Undoable_Action */

public void
Prepare_To_Show()
{
  WindowListener	wl;

  pattern_view.getColumn(PatternTableModel.ID_COLUMN).setWidth(
    pattern_view.getFontMetrics(fixed_width_font).
					stringWidth("########") + 4);
  split_pane.setDividerLocation(328);

  wl = new WindowAdapter() {
    public void windowClosing(WindowEvent e) {
      Confirm_Save_Changes(JOptionPane.YES_NO_OPTION);
      System.exit(0);
    }
  };
  addWindowListener(wl);

} /* Prepare_To_Show */

public void
Reorder_Patterns(ActionEvent e)
{
  String	action;
  boolean	move_up = true;
  int		delta = 0,
		end_row,
		max_y,
		new_y,
		row_cnt = pattern_view.getRowCount(),
		row_from,
		row_to,
		sel_cnt = pattern_view.getSelectedRowCount();
  int []	sel_rows = pattern_view.getSelectedRows();
  Point		corner;
  Dimension	extent;
  String	undo;
  JViewport	vp;

  // Don't move the last (blank) row in the patterns table.
  if (sel_cnt > 0 && sel_rows[sel_cnt - 1] + 1 >= row_cnt)
    --sel_cnt;
  if (sel_cnt < 1)
    return;

  if (e.getSource() instanceof JButton) {
    action = ((JButton)e.getSource()).getToolTipText();
  } else if (e.getSource() instanceof JMenuItem) {
    action = ((JMenuItem)e.getSource()).getText();
  } else {
    return;
  }
  if (action.equals(PatternEditor.REORDER_TOP)) {
    delta = sel_rows[0];
  } else if (action.equals(PatternEditor.REORDER_UP)) {
    if (sel_rows[0] > 0)
      delta = 1;
  } else if (action.equals(PatternEditor.REORDER_DOWN)) {
    move_up = false;
    if (sel_rows[sel_cnt - 1] + 1 < row_cnt - 1)
      delta = 1;
  } else if (action.equals(PatternEditor.REORDER_BOTTOM)) {
    move_up = false;
    delta = row_cnt - sel_rows[sel_cnt - 1] - 2;
  } else {
    return;
  }
  if (delta > 0) {
    undo = "REORDER " + delta + (move_up ? " UP " : " DOWN ") + sel_cnt;

    for (int i = 0; i < sel_cnt; i++)
      undo = undo + " " + i;
    Log_Undoable_Action(undo);

    pattern_view.clearSelection();
    end_row = model.Reorder_Patterns(move_up, sel_cnt, sel_rows, delta);
    if (move_up)
      delta = -delta;
    for (int i = 0; i < sel_cnt; i++)
      pattern_view.addRowSelectionInterval(sel_rows[i] + delta,
      					   sel_rows[i] + delta);
    vp = scroll_list.getViewport();
    corner = vp.getViewPosition();
    extent = vp.getExtentSize();
    if (row_height * end_row < corner.y ||
        row_height * (end_row + 1) > corner.y + extent.height) {
      new_y = row_height * end_row - (extent.height - row_height) / 2;
      if (new_y < 0)
        new_y = 0;
      max_y = pattern_view.getRowCount() * row_height - extent.height;
      if (max_y < 0)
        max_y = 0;
      if (new_y > max_y)
        new_y = max_y;
      vp.setViewPosition(new Point(0, new_y));
    }
  }

} /* Reorder_Patterns */

private int
Save_Changes()
{
  if (model.Is_PatternVector_Untitled())
    return (Save_Changes_As());
  try {
    model.Write_Patterns();
  } catch (IOException e) {
    Set_StatusBar("Unable to write patterns: " + e);
    return (JOptionPane.CANCEL_OPTION);
  }
  Format_Title();
  return (JOptionPane.YES_OPTION);

} /* Save_Changes */

private int
Save_Changes_As()
{
  // Use FileChooser dialog to get output path & file name.
  JFileChooser	chooser = new JFileChooser();
  int		result;

  chooser.setCurrentDirectory(new File(model.Get_Pattern_Path()));
  chooser.setSelectedFile(new File(model.Get_Pattern_Filename()));
  chooser.setDialogTitle(APP_TITLE);
  result = chooser.showSaveDialog(getContentPane());
  if (result == JFileChooser.CANCEL_OPTION)
    return (JOptionPane.CANCEL_OPTION);
  try {
    model.Set_Pattern_Path(chooser.getCurrentDirectory().getName());
    model.Set_Pattern_Filename(chooser.getSelectedFile().getName());
    model.Write_Patterns();
  } catch (IOException e) {
    Set_StatusBar("Unable to write patterns: " + e);
    return (JOptionPane.CANCEL_OPTION);
  }
  Format_Title();
  return (JOptionPane.YES_OPTION);

} /* Save_Changes_As */

public void
Set_StatusBar(String text)
{
  if (status_bar != null)
    status_bar.setText(text);

} /* Set_StatusBar */

private void
Show_About_Dialog()
{
  if (about_dialog == null) {
    JLabel		label,
			title = new JLabel("Pattern Editor 1.0",
				   SwingConstants.CENTER);
    JPanel		about_panel,
			border_panel,
			subpanel = new JPanel(new GridLayout(12, 1));
    JButton		ok_button;
    Dimension		my_size,
			screen_size = getToolkit().getScreenSize();

    about_dialog = new JDialog(pview, "About Pattern Editor", true);
    border_panel = new JPanel();
    about_panel = new JPanel(new BorderLayout());
    title.setFont(title_font);
    title.setForeground(Color.black);
    about_panel.add(title, BorderLayout.NORTH);
    subpanel.add(new JLabel(" "));
    Add_Label("By Dr. Stephen W. Liddle, Assistant Professor",
	      subpanel, subtitle_font);
    Add_Label("and Chad Crawford, Graduate Assistant", subpanel,
	      subtitle_font);
    subpanel.add(new JLabel(" "));
    Add_Label("School of Accountancy and Information Systems",
	      subpanel);
    Add_Label("Marriott School of Management", subpanel);
    Add_Label("Brigham Young University", subpanel);
    subpanel.add(new JLabel(" "));
    Add_Label("A Product of the BYU Data Extraction Research Group",
	      subpanel);
    subpanel.add(new JLabel(" "));
    label = Add_Label("Copyright \u00a9 1998, Board of Trustees " +
		      "of Brigham Young University", subpanel);
    my_size = label.getPreferredSize();
    my_size.width += 5;
    label.setPreferredSize(my_size);
    subpanel.add(new JLabel(" "));
    about_panel.add(subpanel, BorderLayout.CENTER);
    ok_button = new JButton("Close");
    ok_button.setMnemonic('C');
    ok_button.addActionListener( new ActionListener() {
			public void actionPerformed(ActionEvent e) {
			about_dialog.setVisible(false); } } );
    subpanel = new JPanel();
    subpanel.add(ok_button);
    about_panel.add(subpanel, BorderLayout.SOUTH);
    border_panel.add(about_panel);
    border_panel.setBorder(new CompoundBorder(
    				new EmptyBorder(8, 8, 8, 8),
    				new CompoundBorder(new EtchedBorder(),
					new EmptyBorder(5, 5, 5, 5))
				));
    about_dialog.getContentPane().add(border_panel);
    about_dialog.getRootPane().setDefaultButton(ok_button);
    about_dialog.pack();
    my_size = about_dialog.getSize();
    about_dialog.setLocation((screen_size.width - my_size.width) / 2,
			     (screen_size.height - my_size.height) / 2);
    about_dialog.show();
  } else {
    about_dialog.setVisible(true);
  }

} /* Show_About_Dialog */

private void
Show_Help_Dialog()
{
  if (help_dialog == null) {
    JTextComponent	help_editor;
    JPanel		border_panel,
			subpanel = new JPanel(new GridLayout(11, 1));
    Dimension		my_size,
			screen_size = getToolkit().getScreenSize();

    help_dialog = new JDialog(pview, "Pattern Editor Help", true);
    border_panel = new JPanel(new BorderLayout());
    try {
      help_editor = new JEditorPane("file:" + path + "index.html");
    } catch (IOException ex) {
      help_editor = new JTextArea("Unable to get help: " + ex);
    }
    help_editor.setEnabled(true);
    help_editor.setEditable(false);
    border_panel.add(new JScrollPane(help_editor), BorderLayout.CENTER);
    border_panel.setBorder(new EmptyBorder(2, 2, 2, 2));
    help_dialog.getContentPane().add(border_panel);
    help_dialog.setSize(new Dimension(600, 400));
    my_size = help_dialog.getSize();
    help_dialog.setLocation((screen_size.width - my_size.width) / 2,
			    (screen_size.height - my_size.height) / 2);
    help_dialog.addComponentListener(new ComponentAdapter() {
    		public void componentResized(ComponentEvent e) {
			Dimension	hd_size = help_dialog.getSize();
			boolean		too_small = false;

			if (hd_size.width < 200) {
			  hd_size.width = 200;
			  too_small = true;
			}
			if (hd_size.height < 200) {
			  hd_size.height = 200;
			  too_small = true;
			}
			if (too_small)
			  help_dialog.setSize(hd_size);
		}
    	} );
    help_dialog.show();
  } else {
    help_dialog.setVisible(true);
  }

} /* Show_Help_Dialog */

private void
Update_Highlights()
{
  if (!tb_highlight.isSelected()) {
    hlite.removeAllHighlights();
    return;
  }

  int []	rows = pattern_view.getSelectedRows();
  int		row_cnt = pattern_view.getSelectedRowCount();

  hlite.removeAllHighlights();
  if (row_cnt > 0) {
    com.oroinc.text.regex.Pattern		pat;
    com.oroinc.text.regex.PatternMatcherInput	input;
    com.oroinc.text.regex.MatchResult		result;

    if (rows[row_cnt - 1] + 1 >= pattern_view.getRowCount())
      --row_cnt;
    for (int i = row_cnt - 1; i >= 0; i--) {
      try {
        pat = model.Get_Pattern_At(rows[i]).Get_Pattern();
	if (pat != null) {
          input = new com.oroinc.text.regex.PatternMatcherInput(
			  model.Get_Document().getText(0,
			  model.Get_Document().getLength()));
          while (matcher.contains(input, pat)) {
	    result = matcher.getMatch();
	    try {
              hlite.addHighlight(result.beginOffset(0),
			         result.beginOffset(0) + result.length(),
			         painters[rows[i] % NUM_COLORS]);
	    } catch (BadLocationException e) {
	    }
          }
        }
      } catch (BadLocationException e1) {
      } catch (com.oroinc.text.regex.MalformedPatternException e2) {
      }
    }
  }

} /* Update_Highlights */

/*------------------------------------------------------------------------
 *			PUBLIC METHODS
 */
public boolean
Is_Constructed()
{
  return (pe_constructed);

} /* Is_Constructed */

public void
Set_Constructed(boolean val)
{
  pe_constructed = val;

} /* Set_Constructed */

/*========================================================================
 *			INNER CLASSES
 */
/*------------------------------------------------------------------------
 * CLASS PatternCellEditor -- allow setting font on the default cell editor
 */
private class PatternCellEditor extends DefaultCellEditor {

public
PatternCellEditor(JTextField x)
{
  super(x);

} /* PatternCellEditor constructor */

public
PatternCellEditor(JCheckBox x)
{
  super(x);

} /* PatternCellEditor constructor */

public
PatternCellEditor(JComboBox x)
{
  super(x);

} /* PatternCellEditor constructor */

public void
setFont(java.awt.Font f)
{
  if (editorComponent != null) {
    editorComponent.setFont(f);
    editorComponent.setBorder(new EmptyBorder(1, 1, 1, 1));
  }
 
} /* setFont */

} /* class PatternCellEditor */

/*------------------------------------------------------------------------
 * CLASS PatternCellRenderer -- fix border on default cell renderer
 */
private class PatternCellRenderer extends DefaultTableCellRenderer {

private Border my_noFocusBorder;

public
PatternCellRenderer()
{
  super();
  my_noFocusBorder = new EmptyBorder(1, 1, 1, 1);
  setBorder(my_noFocusBorder);

} /* PatternCellRenderer constructor */

public Component
getTableCellRendererComponent(JTable table, Object value,
    			      boolean isSelected, boolean hasFocus,
			      int row, int column) {
  DefaultTableCellRenderer	cr;

  cr = (DefaultTableCellRenderer)super.
  		getTableCellRendererComponent(table, value, isSelected,
					      hasFocus, row, column);
  if (!hasFocus)
    cr.setBorder(my_noFocusBorder);
  return (cr);

} /* getTableCellRendererComponent */

} /* class PatternCellRenderer */

/*------------------------------------------------------------------------
 * CLASS ColorCheckRenderer -- allow setting background color
 */
private class ColorCheckRenderer extends JCheckBox
			       implements TableCellRenderer {
public
ColorCheckRenderer()
{
  super();
  this.setOpaque(true);

} /* ColorCheckRenderer constructor */

public Component
getTableCellRendererComponent(JTable table, Object value,
    			      boolean isSelected, boolean hasFocus,
			      int row, int column) {
  setSelected(value != null && ((Boolean)value).booleanValue());
  this.setBackground(colors[row % NUM_COLORS]);
  return (this);

} /* getTableCellRendererComponent */

} /* class ColorCheckRenderer */

/*------------------------------------------------------------------------
 * CLASS PatternSelection -- Vector of Pattern objects
 *
 * NEEDSWORK: if we want, we can add another couple of DataFlavors to
 *	handle String and plain text clipboard contents.
 */
private class PatternSelection implements Transferable, ClipboardOwner {

private Object		selection;
private DataFlavor	flavor;

public
PatternSelection(Object sel, DataFlavor f)
{
  selection = sel;
  flavor = f;

} /* PatternSelection constructor */

public DataFlavor []
getTransferDataFlavors()
{
  return (new DataFlavor [] { flavor });

} /* getTransferDataFlavors */

public boolean
isDataFlavorSupported(DataFlavor f)
{
  return (flavor.equals(f));

} /* isDataFlavorSupported */

public Object
getTransferData(DataFlavor f) throws UnsupportedFlavorException
{
  if (flavor.equals(f)) {
    return (selection);
  } else {
    throw new UnsupportedFlavorException(f);
  }

} /* getTransferData */

public void
lostOwnership(Clipboard c, Transferable t)
{
  selection = null;

} /* lostOwnership */

} /* class PatternSelection */

/*------------------------------------------------------------------------
 * CLASS PatternTable -- handles events detected by the patterns JTable
 */
private class PatternTable extends JTable {

private PatternEditor	pe;

public
PatternTable(PatternTableModel tm, PatternEditor my_pe)
{
  super(tm);
  pe = my_pe;

} /* PatternTable constructor */

public void
editingStopped(ChangeEvent ev)
{
  PatternCellEditor	ce = null;
  String	undo = "",
		value;
  boolean	sensitive;
  int		col = getEditingColumn(),
		row = getEditingRow();

  if ((ev.getSource() instanceof PatternCellEditor)) {
    ce = (PatternCellEditor)ev.getSource();
    value = ce.getCellEditorValue().toString().trim();
    sensitive = model.Get_Pattern_At(row).Is_Case_Sensitive();
    if (ce.getComponent() instanceof JTextField) {
      JTextField	tf = (JTextField)ce.getComponent();

      tf.setText(value);
    }
  } else {
    JCheckBox	cb = (JCheckBox)
    		     ((DefaultCellEditor)ev.getSource()).getComponent();
    sensitive = cb.isSelected();
    value = model.Get_Pattern_At(row).Get_Regular_Expression();
    model.Set_Case_Sensitivity_At(row, sensitive);
  }
  if ( (col == CASE_COL_NUM) || (col == ID_COL_NUM &&
	!value.equals(model.Get_Pattern_At(row).Get_ID())) ||
       (col == PATTERN_COL_NUM &&
	!value.equals(model.Get_Pattern_At(row).Get_Regular_Expression())) ) {
    if (row + 1 >= model.Get_Num_Patterns()) {
      undo = "INSERT";
      model.Add_Pattern();
    } else {
      undo = "MODIFY";
    }
  }
  if (!undo.equals("")) {
    Format_Title();
    Log_Undoable_Action(undo + " " + row + " " + model.Get_Pattern(row));
  }
  super.editingStopped(ev);

} /* editingStopped */

public void
valueChanged(ListSelectionEvent e)
{
  super.valueChanged(e);
  if (pe != null && pe.Is_Constructed()) {
    Enable_Selected(getSelectedRowCount() > 0);
    Update_Highlights();
  }

} /* valueChanged */

} /* class PatternTable */

/*------------------------------------------------------------------------
 * CLASS ReportTextPane -- disables horizontal scrolling
 */
private class ReportTextPane extends JTextPane {

public
ReportTextPane()
{
  super();

} /* ReportTextPane constructor */

public
ReportTextPane(StyledDocument doc)
{
  super(doc);

} /* ReportTextPane constructor */

public boolean
getScrollableTracksViewportWidth()
{
  return (false);

} /* getScrollableTracksViewportWidth */

} /* class ReportTextPane */

/*========================================================================
 *			CLASS METHODS
 */
public static void
main(String[] args)
{
  PatternEditor	pe;
  Dimension	screen;
  String	pattern_fname = "patterns.txt",
		report_fname = "report.txt";

  try {
    UIManager.setLookAndFeel(UIManager.
    			getCrossPlatformLookAndFeelClassName());
  } catch (Exception e) {
  }

  if (args.length > 2) {
    System.out.println("Usage: java PatternEditor [patterns_filename " +
    		       "[report_filename]]");
    System.exit(-1);
  }
  if (args.length > 1) {
    report_fname = args[1];
  }
  if (args.length > 0) {
    pattern_fname = args[0];
  }

  screen = Toolkit.getDefaultToolkit().getScreenSize();
  pe = new PatternEditor(pattern_fname, report_fname);
  pe.pack();
  pe.setSize(new Dimension( Math.min(1024, screen.width),
  			    Math.min(768, screen.height) ));
  pe.Prepare_To_Show();
  pe.setLocation(screen.width / 2 - pe.getSize().width / 2,
		 screen.height / 2 - pe.getSize().height / 2);
  pe.Set_Constructed(true);
  pe.show();

} /* main */

} /* class PatternEditor */

/*========================================================================
 *			END OF FILE $RCSfile: PatternEditor.java,v $
 */
