/*========================================================================
 * $RCSfile: PatternTableModel.java,v $
 *
 * $Author: liddle $
 * $Revision: 1.1 $
 * $Date: 1998/08/06 16:29:37 $
 *
 * $Log: PatternTableModel.java,v $
 * Revision 1.1  1998/08/06  16:29:37  liddle
 * Initial revision
 *
 *------------------------------------------------------------------------
 * Pattern Editor
 * Data Extraction Research Group
 * School of Accountancy and Information Systems
 * Marriott School of Management
 * 540 TNRB
 * Brigham Young University
 * Provo, UT 84602-3068
 * liddle@byu.edu
 *
 * Copyright (C) 1998, Board of Trustees of Brigham Young University
 *
 * Pattern Editor software, both binary and source (hereafter, Software)
 * is copyrighted by The Board of Trustees of Brigham Young University
 * (BYU), and ownership remains with BYU.
 *
 * BYU grants you (hereafter, Licensee) a license to use the Software
 * for academic, research and internal business purposes only, without a
 * fee.  Licensee may distribute the binary and source code (if released)
 * to third parties provided that the copyright notice and this statement
 * appears on all copies and that no charge is associated with such
 * copies.
 *
 * Licensee may not make derivative works without first arranging an
 * additional license agreement with BYU.
 *
 * Any Licensee wishing to make commercial use of the Software should
 * contact BYU to negotiate an appropriate license for such
 * commercial use.  Commercial use includes (1) integration of all or
 * part of the source code into a product for sale or license by or on
 * behalf of Licensee to third parties, or (2) distribution of the binary
 * code or source code to third parties that need it to utilize a
 * commercial product sold or licensed by or on behalf of Licensee.
 *
 * BYU MAKES NO REPRESENTATIONS ABOUT THE SUITABILITY OF THIS SOFTWARE
 * FOR ANY PURPOSE.  IT IS PROVIDED "AS IS" WITHOUT EXPRESS OR IMPLIED
 * WARRANTY.  BYU SHALL NOT BE LIABLE FOR ANY DAMAGES SUFFERED BY THE
 * USERS OF THIS SOFTWARE.
 *
 * By using or copying this Software, Licensee agrees to abide by the
 * copyright law and all other applicable laws of the United States
 * including, but not limited to, export control laws, and the terms
 * of this license.  BYU shall have the right to terminate this license
 * immediately by written notice upon Licensee's breach of, or
 * non-compliance with, any of its terms.  Licensee may be held legally
 * responsible for any copyright infringement that is caused or
 * encouraged by Licensee's failure to abide by the terms of this license.
 *
 * Comments and questions are welcome and can be sent to liddle@byu.edu.
 */
import java.io.*;
import java.util.*;

import com.sun.java.swing.*;
import com.sun.java.swing.event.*;
import com.sun.java.swing.table.*;

/*------------------------------------------------------------------------
 * CLASS PatternTableModel -- handles model for our patterns table
 */
public class PatternTableModel extends AbstractTableModel {

/*------------------------------------------------------------------------
 *			PUBLIC CONSTANTS
 */
public static final String	CASE_COLUMN = "Case Sensitive",
				ID_COLUMN = "ID",
				PATTERN_COLUMN = "Pattern";

/*------------------------------------------------------------------------
 *			PRIVATE CONSTANTS
 */
private static final int	CASE_COL_NUM = 0,
				ID_COL_NUM = 1,
				PATTERN_COL_NUM = 2,
				NUM_COLUMNS = 3;

/*------------------------------------------------------------------------
 *			ATTRIBUTES
 */
private PatternVector	patterns;	// The main Vector of patterns

/*========================================================================
 *			PUBLIC METHODS
 */
/*------------------------------------------------------------------------
 *			CONSTRUCTORS
 */
public
PatternTableModel()
{
  super();
  patterns = new PatternVector();

} /* PatternTableModel constructor */

/*------------------------------------------------------------------------
 *			ACCESSORS
 */
public Class
getColumnClass(int col)
{
  return (getValueAt(0, col).getClass());

} /* getColumnClass */

public int
getColumnCount()
{
  return (NUM_COLUMNS);

} /* getColumnCount */

public String
getColumnName(int col)
{
  switch (col) {
    case CASE_COL_NUM:
      return (CASE_COLUMN);
    case ID_COL_NUM:
      return (ID_COLUMN);
  }
  return (PATTERN_COLUMN);

} /* getColumnName */

public PatternVector
Get_Patterns()
{
  return (patterns);

} /* Get_Patterns */

public int
getRowCount()
{
  return (patterns.size());

} /* getRowCount */

public Object
getValueAt(int row, int col)
{
  switch (col) {
    case CASE_COL_NUM:
      return (patterns.Get(row).Get_Case_Sensitive());
    case ID_COL_NUM:
      return (patterns.Get(row).Get_ID());
  }
  return (patterns.Get(row).Get_Regular_Expression());

} /* getValueAt */

public boolean
isCellEditable(int row, int col)
{
  return (true);

} /* isCellEditable */

/*------------------------------------------------------------------------
 *			MUTATORS
 */
public void
Add_Pattern()
{
  patterns.Add_Pattern();
  fireTableDataChanged();

} /* Add_Pattern */

public void
setValueAt(Object aValue, int row, int col)
{
  switch (col) {
    case CASE_COL_NUM:
      patterns.Get(row).Set_Case_Sensitivity((Boolean)aValue);
      break;
    case ID_COL_NUM:
      patterns.Get(row).Set_ID((String)aValue);
      break;
    case PATTERN_COL_NUM:
      patterns.Get(row).Set_Regular_Expression((String)aValue);
      break;
  }
  fireTableCellUpdated(row, col);

} /* setValueAt */

/*------------------------------------------------------------------------
 *			FACILITATORS
 */
public void
fireTableDataChanged()
{
  fireTableChanged(new TableModelEvent(this)); 

} /* fireTableDataChanged */

public void
Insert_Patterns(PatternVector pats, int row)
{
  for (int i = pats.size() - 1; i >= 0; i--)
    patterns.insertElementAt(pats.Get(i).clone(), row);
  fireTableDataChanged();

} /* Insert_Patterns */

public boolean
Load_Patterns(String path)
{
  boolean			error = false,
  				sensitive;
  String			line;
  BufferedReader		fin;
  com.oroinc.text.regex.Pattern	pattern;

  patterns.removeAllElements();
  if (!path.equals(PatternEditorModel.UNTITLED_NAME)) {
    try {
      fin = new BufferedReader(new FileReader(path));
      line = fin.readLine();
      while (line != null) {
	patterns.Add_Pattern(line);
        line = fin.readLine();
      }
      fin.close();
    } catch (Exception e) {
      patterns.removeAllElements();
      error = true;
    }
  }
  patterns.Add_Pattern();
  fireTableDataChanged();
  return (error);

} /* Load_Patterns */

public void
Remove_Patterns(int [] sel_rows, int sel_cnt)
{
  boolean	deleted_last;

  deleted_last = (sel_rows[sel_cnt - 1] + 1 >= patterns.size());
  for (int i = sel_cnt - 1; i >= 0; i--)
    patterns.removeElementAt(sel_rows[i]);
  if (deleted_last)
    patterns.Add_Pattern();
  fireTableDataChanged();

} /* Remove_Patterns */

public int
Reorder_Patterns( boolean move_up, int sel_cnt,
		  int [] sel_rows, int delta )
{
  int	last_sel = 0,
  	row_from,
	row_to;

  if (move_up) {
    for (int i = 0; i < sel_cnt; i++) {
      row_from = sel_rows[i];
      row_to = row_from - delta;
      patterns.Move_Pattern(row_from, row_to);
    }
    last_sel = sel_rows[0] - delta;
  } else {
    for (int i = sel_cnt - 1; i >= 0; i--) {
      row_from = sel_rows[i];
      row_to = row_from + delta;
      patterns.Move_Pattern(row_from, row_to);
    }
    last_sel = sel_rows[sel_cnt - 1] + delta;
  }
  fireTableChanged(new TableModelEvent(this));
  return (last_sel);

} /* Reorder_Patterns */

} /* class PatternTableModel */

/*========================================================================
 *			END OF FILE $RCSfile: PatternTableModel.java,v $
 */
